@@ -25,7 +25,7 @@ PORTS.forEach(function(port) {
 
 var tunnel = new BrowserStackTunnel({
   key: ACCESS_KEY,
-  tunnelIdentifier: TUNNEL_IDENTIFIER,
+  localIdentifier: TUNNEL_IDENTIFIER,
   hosts: hosts
 });
 
