@@ -4,6 +4,7 @@ exports.config = {
   allScriptsTimeout: 11000,
 
   specs: [
+    'test/e2e/tests/**/*.js',
     'build/docs/ptore2e/**/*.js',
     'docs/app/e2e/*.scenario.js'
   ],
@@ -12,9 +13,9 @@ exports.config = {
     'browserName': 'chrome'
   },
 
-  baseUrl: 'http://localhost:8000/build/docs/',
+  baseUrl: 'http://localhost:8000/',
 
-  framework: 'jasmine',
+  framework: 'jasmine2',
 
   onPrepare: function() {
     /* global angular: false, browser: false, jasmine: false */
@@ -28,9 +29,10 @@ exports.config = {
 
     browser.addMockModule('disableNgAnimate', disableNgAnimate);
 
-    require('jasmine-reporters');
-    jasmine.getEnv().addReporter(
-      new jasmine.JUnitXmlReporter('test_out/e2e-' + exports.config.capabilities.browserName + '-', true, true));
+    var reporters = require('jasmine-reporters');
+    jasmine.getEnv().addReporter(new reporters.JUnitXmlReporter({
+      savePath: 'test_out/docs-e2e-' + exports.config.capabilities.browserName + '-'
+    }));
   },
 
   jasmineNodeOpts: {
