@@ -59,7 +59,7 @@ function capabilitiesForBrowserStack(capabilities) {
     'browserstack.tunnelIdentifier': process.env.TRAVIS_JOB_NUMBER,
     'tunnelIdentifier': process.env.TRAVIS_JOB_NUMBER,
 
-    'name': 'Angular E2E',
+    'name': 'AngularJS E2E',
     'build': process.env.TRAVIS_BUILD_NUMBER,
 
     'browserName': capabilities.browserName,
@@ -73,12 +73,14 @@ function capabilitiesForSauceLabs(capabilities) {
   return {
     'tunnel-identifier': process.env.TRAVIS_JOB_NUMBER,
 
-    'name': 'Angular E2E',
+    'name': 'AngularJS E2E',
     'build': process.env.TRAVIS_BUILD_NUMBER,
 
     'browserName': capabilities.browserName,
     'platform': capabilities.platform,
     'version': capabilities.version,
-    'elementScrollBehavior': 1
+    'elementScrollBehavior': 1,
+    // Allow e2e test sessions to run for a maximum of 35 minutes, instead of the default 30 minutes.
+    'maxDuration': 2100
   };
 }
