@@ -17,7 +17,7 @@ ARG_DEFS=(
 )
 
 function checkVersionNumber() {
-  BRANCH_PATTERN=$(readJsonProp "package.json" "branchVersion")
+  BRANCH_PATTERN=$(readJsonProp "package.json" "branchPattern")
   if [[ $VERSION_NUMBER != $BRANCH_PATTERN ]]; then
     echo "version-number needs to match $BRANCH_PATTERN on this branch"
     usage
