@@ -7,5 +7,5 @@ for FILE in $LOG_FILES; do
   echo "================================================================================"
   echo " $FILE"
   echo "================================================================================"
-  cat $FILE
+  cat $FILE || true
 done
