@@ -4,18 +4,27 @@ var PLURAL_CATEGORY = {ZERO: "zero", ONE: "one", TWO: "two", FEW: "few", MANY: "
 $provide.value("$locale", {
   "DATETIME_FORMATS": {
     "AMPMS": [
-      "am",
-      "pm"
+      "AM",
+      "PM"
     ],
     "DAY": [
       "\u09b0\u09ac\u09bf\u09ac\u09be\u09b0",
       "\u09b8\u09cb\u09ae\u09ac\u09be\u09b0",
       "\u09ae\u0999\u09cd\u0997\u09b2\u09ac\u09be\u09b0",
       "\u09ac\u09c1\u09a7\u09ac\u09be\u09b0",
-      "\u09ac\u09c3\u09b9\u09b7\u09cd\u09aa\u09a4\u09bf\u09ac\u09be\u09b0",
+      "\u09ac\u09c3\u09b9\u09b8\u09cd\u09aa\u09a4\u09bf\u09ac\u09be\u09b0",
       "\u09b6\u09c1\u0995\u09cd\u09b0\u09ac\u09be\u09b0",
       "\u09b6\u09a8\u09bf\u09ac\u09be\u09b0"
     ],
+    "ERANAMES": [
+      "\u0996\u09cd\u09b0\u09bf\u09b8\u09cd\u099f\u09aa\u09c2\u09b0\u09cd\u09ac",
+      "\u0996\u09c3\u09b7\u09cd\u099f\u09be\u09ac\u09cd\u09a6"
+    ],
+    "ERAS": [
+      "\u0996\u09cd\u09b0\u09bf\u09b8\u09cd\u099f\u09aa\u09c2\u09b0\u09cd\u09ac",
+      "\u0996\u09c3\u09b7\u09cd\u099f\u09be\u09ac\u09cd\u09a6"
+    ],
+    "FIRSTDAYOFWEEK": 6,
     "MONTH": [
       "\u099c\u09be\u09a8\u09c1\u09af\u09bc\u09be\u09b0\u09c0",
       "\u09ab\u09c7\u09ac\u09cd\u09b0\u09c1\u09af\u09bc\u09be\u09b0\u09c0",
@@ -40,6 +49,20 @@ $provide.value("$locale", {
       "\u09b6\u09a8\u09bf"
     ],
     "SHORTMONTH": [
+      "\u099c\u09be\u09a8\u09c1",
+      "\u09ab\u09c7\u09ac",
+      "\u09ae\u09be\u09b0\u09cd\u099a",
+      "\u098f\u09aa\u09cd\u09b0\u09bf\u09b2",
+      "\u09ae\u09c7",
+      "\u099c\u09c1\u09a8",
+      "\u099c\u09c1\u09b2\u09be\u0987",
+      "\u0986\u0997\u09b8\u09cd\u099f",
+      "\u09b8\u09c7\u09aa\u09cd\u099f\u09c7\u09ae\u09cd\u09ac\u09b0",
+      "\u0985\u0995\u09cd\u099f\u09cb\u09ac\u09b0",
+      "\u09a8\u09ad\u09c7\u09ae\u09cd\u09ac\u09b0",
+      "\u09a1\u09bf\u09b8\u09c7\u09ae\u09cd\u09ac\u09b0"
+    ],
+    "STANDALONEMONTH": [
       "\u099c\u09be\u09a8\u09c1\u09af\u09bc\u09be\u09b0\u09c0",
       "\u09ab\u09c7\u09ac\u09cd\u09b0\u09c1\u09af\u09bc\u09be\u09b0\u09c0",
       "\u09ae\u09be\u09b0\u09cd\u099a",
@@ -53,6 +76,10 @@ $provide.value("$locale", {
       "\u09a8\u09ad\u09c7\u09ae\u09cd\u09ac\u09b0",
       "\u09a1\u09bf\u09b8\u09c7\u09ae\u09cd\u09ac\u09b0"
     ],
+    "WEEKENDRANGE": [
+      6,
+      6
+    ],
     "fullDate": "EEEE, d MMMM, y",
     "longDate": "d MMMM, y",
     "medium": "d MMM, y h:mm:ss a",
@@ -92,6 +119,7 @@ $provide.value("$locale", {
     ]
   },
   "id": "bn-in",
-  "pluralCat": function (n, opt_precision) {  var i = n | 0;  if (i == 0 || n == 1) {    return PLURAL_CATEGORY.ONE;  }  return PLURAL_CATEGORY.OTHER;}
+  "localeID": "bn_IN",
+  "pluralCat": function(n, opt_precision) {  var i = n | 0;  if (i == 0 || n == 1) {    return PLURAL_CATEGORY.ONE;  }  return PLURAL_CATEGORY.OTHER;}
 });
-}]);
\ No newline at end of file
+}]);
