@@ -21,8 +21,8 @@ $provide.value("$locale", {
       "\uc11c\uae30"
     ],
     "ERAS": [
-      "\uae30\uc6d0\uc804",
-      "\uc11c\uae30"
+      "BC",
+      "AD"
     ],
     "FIRSTDAYOFWEEK": 0,
     "MONTH": [
@@ -108,8 +108,8 @@ $provide.value("$locale", {
       {
         "gSize": 3,
         "lgSize": 3,
-        "maxFrac": 2,
-        "minFrac": 2,
+        "maxFrac": 0,
+        "minFrac": 0,
         "minInt": 1,
         "negPre": "-\u00a4",
         "negSuf": "",
