@@ -26,13 +26,13 @@ $provide.value("$locale", {
       "\u042d\u041a"
     ],
     "DAY": [
-      "\u0411\u0430\u0441\u043a\u044b\u04bb\u044b\u0430\u043d\u043d\u044c\u0430",
-      "\u0411\u044d\u043d\u0438\u0434\u0438\u044d\u043b\u0438\u043d\u043d\u044c\u0438\u043a",
-      "\u041e\u043f\u0442\u0443\u043e\u0440\u0443\u043d\u043d\u044c\u0443\u043a",
-      "\u0421\u044d\u0440\u044d\u0434\u044d",
-      "\u0427\u044d\u043f\u043f\u0438\u044d\u0440",
+      "\u0431\u0430\u0441\u043a\u044b\u04bb\u044b\u0430\u043d\u043d\u044c\u0430",
+      "\u0431\u044d\u043d\u0438\u0434\u0438\u044d\u043d\u043d\u044c\u0438\u043a",
+      "\u043e\u043f\u0442\u0443\u043e\u0440\u0443\u043d\u043d\u044c\u0443\u043a",
+      "\u0441\u044d\u0440\u044d\u0434\u044d",
+      "\u0447\u044d\u043f\u043f\u0438\u044d\u0440",
       "\u0411\u044d\u044d\u0442\u0438\u04a5\u0441\u044d",
-      "\u0421\u0443\u0431\u0443\u043e\u0442\u0430"
+      "\u0441\u0443\u0431\u0443\u043e\u0442\u0430"
     ],
     "ERANAMES": [
       "\u0431. \u044d. \u0438.",
@@ -55,44 +55,44 @@ $provide.value("$locale", {
       "\u0411\u0430\u043b\u0430\u0495\u0430\u043d \u044b\u0439\u044b\u043d",
       "\u0410\u043b\u0442\u044b\u043d\u043d\u044c\u044b",
       "\u0421\u044d\u0442\u0438\u043d\u043d\u044c\u0438",
-      "\u0410\u0445\u0441\u044b\u043d\u043d\u044c\u044b"
+      "\u0430\u0445\u0441\u044b\u043d\u043d\u044c\u044b"
     ],
     "SHORTDAY": [
-      "\u0411\u0441",
-      "\u0411\u043d",
-      "\u041e\u043f",
-      "\u0421\u044d",
-      "\u0427\u043f",
-      "\u0411\u044d",
-      "\u0421\u0431"
+      "\u0431\u0441",
+      "\u0431\u043d",
+      "\u043e\u043f",
+      "\u0441\u044d",
+      "\u0447\u043f",
+      "\u0431\u044d",
+      "\u0441\u0431"
     ],
     "SHORTMONTH": [
       "\u0422\u043e\u0445\u0441",
       "\u041e\u043b\u0443\u043d",
-      "\u041a\u043b\u043d_\u0442\u0442\u0440",
-      "\u041c\u0443\u0441_\u0443\u0441\u0442",
-      "\u042b\u0430\u043c_\u0439\u043d",
-      "\u0411\u044d\u0441_\u0439\u043d",
-      "\u041e\u0442_\u0439\u043d",
-      "\u0410\u0442\u0440\u0434\u044c_\u0439\u043d",
-      "\u0411\u043b\u0495\u043d_\u0439\u043d",
+      "\u041a\u043b\u043d",
+      "\u041c\u0441\u0443",
+      "\u042b\u0430\u043c",
+      "\u0411\u044d\u0441",
+      "\u041e\u0442\u0439",
+      "\u0410\u0442\u0440",
+      "\u0411\u043b\u0495",
       "\u0410\u043b\u0442",
       "\u0421\u044d\u0442",
       "\u0410\u0445\u0441"
     ],
     "STANDALONEMONTH": [
-      "\u0422\u043e\u0445\u0441\u0443\u043d\u043d\u044c\u0443",
-      "\u041e\u043b\u0443\u043d\u043d\u044c\u0443",
-      "\u041a\u0443\u043b\u0443\u043d \u0442\u0443\u0442\u0430\u0440",
-      "\u041c\u0443\u0443\u0441 \u0443\u0441\u0442\u0430\u0440",
-      "\u042b\u0430\u043c \u044b\u0439\u044b\u043d",
-      "\u0411\u044d\u0441 \u044b\u0439\u044b\u043d",
-      "\u041e\u0442 \u044b\u0439\u044b\u043d",
-      "\u0410\u0442\u044b\u0440\u0434\u044c\u044b\u0445 \u044b\u0439\u044b\u043d",
-      "\u0411\u0430\u043b\u0430\u0495\u0430\u043d \u044b\u0439\u044b\u043d",
-      "\u0410\u043b\u0442\u044b\u043d\u043d\u044c\u044b",
-      "\u0421\u044d\u0442\u0438\u043d\u043d\u044c\u0438",
-      "\u0410\u0445\u0441\u044b\u043d\u043d\u044c\u044b"
+      "\u0442\u043e\u0445\u0441\u0443\u043d\u043d\u044c\u0443",
+      "\u043e\u043b\u0443\u043d\u043d\u044c\u0443",
+      "\u043a\u0443\u043b\u0443\u043d \u0442\u0443\u0442\u0430\u0440",
+      "\u043c\u0443\u0443\u0441 \u0443\u0441\u0442\u0430\u0440",
+      "\u044b\u0430\u043c \u044b\u0439\u0430",
+      "\u0431\u044d\u0441 \u044b\u0439\u0430",
+      "\u043e\u0442 \u044b\u0439\u0430",
+      "\u0430\u0442\u044b\u0440\u0434\u044c\u044b\u0445 \u044b\u0439\u0430",
+      "\u0431\u0430\u043b\u0430\u0495\u0430\u043d \u044b\u0439\u0430",
+      "\u0430\u043b\u0442\u044b\u043d\u043d\u044c\u044b",
+      "\u0441\u044d\u0442\u0438\u043d\u043d\u044c\u0438",
+      "\u0430\u0445\u0441\u044b\u043d\u043d\u044c\u044b"
     ],
     "WEEKENDRANGE": [
       5,
@@ -108,9 +108,9 @@ $provide.value("$locale", {
     "shortTime": "HH:mm"
   },
   "NUMBER_FORMATS": {
-    "CURRENCY_SYM": "\u0440\u0443\u0431.",
-    "DECIMAL_SEP": ".",
-    "GROUP_SEP": ",",
+    "CURRENCY_SYM": "\u20bd",
+    "DECIMAL_SEP": ",",
+    "GROUP_SEP": "\u00a0",
     "PATTERNS": [
       {
         "gSize": 3,
@@ -129,10 +129,10 @@ $provide.value("$locale", {
         "maxFrac": 2,
         "minFrac": 2,
         "minInt": 1,
-        "negPre": "-\u00a4\u00a0",
-        "negSuf": "",
-        "posPre": "\u00a4\u00a0",
-        "posSuf": ""
+        "negPre": "-",
+        "negSuf": "\u00a0\u00a4",
+        "posPre": "",
+        "posSuf": "\u00a0\u00a4"
       }
     ]
   },
