@@ -230,6 +230,29 @@ describe('validators', function() {
       expect(ctrl.$error.pattern).toBe(true);
       expect(ctrlNg.$error.pattern).toBe(true);
     }));
+
+    it('should only validate once after compilation when inside ngRepeat', function() {
+
+      $rootScope.pattern = /\d{4}/;
+
+      helper.compileInput(
+         '<div ng-repeat="input in [0]">' +
+           '<input type="text" ng-model="value" pattern="\\d{4}" validation-spy="pattern" />' +
+         '</div>');
+
+      $rootScope.$digest();
+
+      expect(helper.validationCounter.pattern).toBe(1);
+
+      helper.compileInput(
+         '<div ng-repeat="input in [0]">' +
+           '<input type="text" ng-model="value" ng-pattern="pattern" validation-spy="pattern" />' +
+         '</div>');
+
+      $rootScope.$digest();
+
+      expect(helper.validationCounter.pattern).toBe(1);
+    });
   });
 
 
@@ -312,9 +335,31 @@ describe('validators', function() {
       expect(ctrl.$error.minlength).toBe(true);
       expect(ctrlNg.$error.minlength).toBe(true);
     }));
-  });
 
 
+    it('should only validate once after compilation when inside ngRepeat', function() {
+      $rootScope.minlength = 5;
+
+      var element = helper.compileInput(
+         '<div ng-repeat="input in [0]">' +
+           '<input type="text" ng-model="value" minlength="{{minlength}}" validation-spy="minlength" />' +
+         '</div>');
+
+      $rootScope.$digest();
+
+      expect(helper.validationCounter.minlength).toBe(1);
+
+      element = helper.compileInput(
+         '<div ng-repeat="input in [0]">' +
+           '<input type="text" ng-model="value" ng-minlength="minlength" validation-spy="minlength" />' +
+         '</div>');
+
+      $rootScope.$digest();
+
+      expect(helper.validationCounter.minlength).toBe(1);
+    });
+  });
+
   describe('maxlength', function() {
 
     it('should invalidate values that are longer than the given maxlength', function() {
@@ -500,6 +545,29 @@ describe('validators', function() {
       expect(ctrl.$error.maxlength).toBe(true);
       expect(ctrlNg.$error.maxlength).toBe(true);
     }));
+
+
+    it('should only validate once after compilation when inside ngRepeat', function() {
+      $rootScope.maxlength = 5;
+
+      var element = helper.compileInput(
+         '<div ng-repeat="input in [0]">' +
+           '<input type="text" ng-model="value" maxlength="{{maxlength}}" validation-spy="maxlength" />' +
+         '</div>');
+
+      $rootScope.$digest();
+
+      expect(helper.validationCounter.maxlength).toBe(1);
+
+      element = helper.compileInput(
+         '<div ng-repeat="input in [0]">' +
+           '<input type="text" ng-model="value" ng-maxlength="maxlength" validation-spy="maxlength" />' +
+         '</div>');
+
+      $rootScope.$digest();
+
+      expect(helper.validationCounter.maxlength).toBe(1);
+    });
   });
 
 
@@ -626,5 +694,73 @@ describe('validators', function() {
       expect(ctrl.$error.required).toBe(true);
       expect(ctrlNg.$error.required).toBe(true);
     }));
+
+
+    it('should override "required" when ng-required="false" is set', function() {
+      var inputElm = helper.compileInput('<input type="text" ng-model="notDefined" required ng-required="false" />');
+
+      expect(inputElm).toBeValid();
+    });
+
+
+    it('should validate only once after compilation when inside ngRepeat', function() {
+      helper.compileInput(
+         '<div ng-repeat="input in [0]">' +
+           '<input type="text" ng-model="value" required validation-spy="required" />' +
+         '</div>');
+
+      $rootScope.$digest();
+
+      expect(helper.validationCounter.required).toBe(1);
+    });
+
+
+    it('should validate only once after compilation when inside ngRepeat and ngRequired is true', function() {
+      $rootScope.isRequired = true;
+
+      helper.compileInput(
+         '<div ng-repeat="input in [0]">' +
+           '<input type="text" ng-model="value" ng-required="isRequired" validation-spy="required" />' +
+         '</div>');
+
+      expect(helper.validationCounter.required).toBe(1);
+    });
+
+
+    it('should validate only once after compilation when inside ngRepeat and ngRequired is false', function() {
+      $rootScope.isRequired = false;
+
+      helper.compileInput(
+         '<div ng-repeat="input in [0]">' +
+           '<input type="text" ng-model="value" ng-required="isRequired" validation-spy="required" />' +
+         '</div>');
+
+      expect(helper.validationCounter.required).toBe(1);
+    });
+
+
+    it('should validate once when inside ngRepeat, and set the "required" error when ngRequired is false by default', function() {
+      $rootScope.isRequired = false;
+      $rootScope.refs = {};
+
+      var elm = helper.compileInput(
+        '<div ng-repeat="input in [0]">' +
+          '<input type="text" ng-ref="refs.input" ng-ref-read="ngModel" ng-model="value" ng-required="isRequired" validation-spy="required" />' +
+        '</div>');
+
+      expect(helper.validationCounter.required).toBe(1);
+      expect($rootScope.refs.input.$error.required).toBeUndefined();
+    });
+
+
+    it('should validate only once when inside ngIf with required on non-input elements', inject(function($compile) {
+      $rootScope.value = '12';
+      $rootScope.refs = {};
+      helper.compileInput('<div ng-if="true"><span ng-model="value" ng-ref="refs.ctrl" ng-ref-read="ngModel" required validation-spy="required"></span></div>');
+      $rootScope.$digest();
+
+      expect(helper.validationCounter.required).toBe(1);
+      expect($rootScope.refs.ctrl.$error.required).not.toBe(true);
+    }));
   });
 });
