@@ -1,5 +1,7 @@
 'use strict';
 
+/* eslint-disable no-script-url */
+
 describe('$interpolate', function() {
 
   it('should return the interpolation object when there are no bindings and textOnly is undefined',
@@ -35,6 +37,29 @@ describe('$interpolate', function() {
     expect($interpolate('{{ false }}')({})).toEqual('false');
   }));
 
+  it('should use custom toString when present', inject(function($interpolate, $rootScope) {
+    var context = {
+      a: {
+        toString: function() {
+          return 'foo';
+        }
+      }
+    };
+
+    expect($interpolate('{{ a }}')(context)).toEqual('foo');
+  }));
+
+  it('should NOT use toString on array objects', inject(function($interpolate) {
+    expect($interpolate('{{a}}')({ a: [] })).toEqual('[]');
+  }));
+
+
+  it('should NOT use toString on Date objects', inject(function($interpolate) {
+    var date = new Date(2014, 10, 10);
+    expect($interpolate('{{a}}')({ a: date })).toBe(JSON.stringify(date));
+    expect($interpolate('{{a}}')({ a: date })).not.toEqual(date.toString());
+  }));
+
 
   it('should return interpolation function', inject(function($interpolate, $rootScope) {
     var interpolateFn = $interpolate('Hello {{name}}!');
@@ -126,6 +151,22 @@ describe('$interpolate', function() {
       expect($rootScope.$countWatchers()).toBe(0);
     }));
 
+    it('should respect one-time bindings for literals', inject(function($interpolate, $rootScope) {
+      var calls = [];
+      $rootScope.$watch($interpolate('{{ ::{x: x} }}'), function(val) {
+        calls.push(val);
+      });
+
+      $rootScope.$apply();
+      expect(calls.pop()).toBe('{}');
+
+      $rootScope.$apply('x = 1');
+      expect(calls.pop()).toBe('{"x":1}');
+
+      $rootScope.$apply('x = 2');
+      expect(calls.pop()).toBeUndefined();
+    }));
+
     it('should stop watching strings with no expressions after first execution',
       inject(function($interpolate, $rootScope) {
         var spy = jasmine.createSpy();
@@ -228,7 +269,9 @@ describe('$interpolate', function() {
 
       expect(function() {
         $interpolate('{{foo}}', true, sce.CSS)(scope);
-      }).toThrowMinErr('$interpolate', 'interr');
+      }).toThrowMinErr(
+          '$interpolate', 'interr', 'Can\'t interpolate: {{foo}}\nError: [$sce:unsafe] ' +
+          'Attempting to use an unsafe value in a safe context.');
     }));
 
     it('should NOT interpolate mistyped expressions', inject(function($interpolate, $rootScope) {
@@ -237,7 +280,9 @@ describe('$interpolate', function() {
 
       expect(function() {
         $interpolate('{{foo}}', true, sce.HTML)(scope);
-      }).toThrowMinErr('$interpolate', 'interr');
+      }).toThrowMinErr(
+          '$interpolate', 'interr', 'Can\'t interpolate: {{foo}}\nError: [$sce:unsafe] ' +
+          'Attempting to use an unsafe value in a safe context.');
     }));
 
     it('should interpolate trusted expressions in a regular context', inject(function($interpolate) {
@@ -252,17 +297,16 @@ describe('$interpolate', function() {
 
     // The concatenation of trusted values does not necessarily result in a trusted value.  (For
     // instance, you can construct evil JS code by putting together pieces of JS strings that are by
-    // themselves safe to execute in isolation.)
+    // themselves safe to execute in isolation). Therefore, some contexts disable it, such as CSS.
     it('should NOT interpolate trusted expressions with multiple parts', inject(function($interpolate) {
       var foo = sce.trustAsCss('foo');
       var bar = sce.trustAsCss('bar');
       expect(function() {
         return $interpolate('{{foo}}{{bar}}', true, sce.CSS)({foo: foo, bar: bar});
       }).toThrowMinErr(
-                '$interpolate', 'noconcat', 'Error while interpolating: {{foo}}{{bar}}\n' +
+                '$interpolate', 'interr', 'Error while interpolating: {{foo}}{{bar}}\n' +
                 'Strict Contextual Escaping disallows interpolations that concatenate multiple ' +
-                'expressions when a trusted value is required.  See ' +
-                'http://docs.angularjs.org/api/ng.$sce');
+                'expressions when a trusted value is required.  See http://docs.angularjs.org/api/ng.$sce');
     }));
   });
 
@@ -341,26 +385,32 @@ describe('$interpolate', function() {
 
 
   describe('isTrustedContext', function() {
-    it('should NOT interpolate a multi-part expression when isTrustedContext is true', inject(function($interpolate) {
-      var isTrustedContext = true;
+    it('should NOT interpolate a multi-part expression when isTrustedContext is RESOURCE_URL', inject(function($sce, $interpolate) {
+      var isTrustedContext = $sce.RESOURCE_URL;
       expect(function() {
-          $interpolate('constant/{{var}}', true, isTrustedContext);
+          $interpolate('constant/{{var}}', true, isTrustedContext)('val');
         }).toThrowMinErr(
-            '$interpolate', 'noconcat', 'Error while interpolating: constant/{{var}}\nStrict ' +
-            'Contextual Escaping disallows interpolations that concatenate multiple expressions ' +
-            'when a trusted value is required.  See http://docs.angularjs.org/api/ng.$sce');
+            '$interpolate', 'interr',
+            'Can\'t interpolate: constant/{{var}}\nError: [$interpolate:noconcat] Error while ' +
+            'interpolating: constant/{{var}}\nStrict Contextual Escaping disallows interpolations ' +
+            'that concatenate multiple expressions when a trusted value is required.  ' +
+            'See http://docs.angularjs.org/api/ng.$sce');
       expect(function() {
-        $interpolate('{{var}}/constant', true, isTrustedContext);
+        $interpolate('{{var}}/constant', true, isTrustedContext)('val');
       }).toThrowMinErr(
-          '$interpolate', 'noconcat', 'Error while interpolating: {{var}}/constant\nStrict ' +
-            'Contextual Escaping disallows interpolations that concatenate multiple expressions ' +
-            'when a trusted value is required.  See http://docs.angularjs.org/api/ng.$sce');
-      expect(function() {
-          $interpolate('{{foo}}{{bar}}', true, isTrustedContext);
+          '$interpolate', 'interr',
+            'Can\'t interpolate: {{var}}/constant\nError: [$interpolate:noconcat] Error while ' +
+            'interpolating: {{var}}/constant\nStrict Contextual Escaping disallows interpolations ' +
+            'that concatenate multiple expressions when a trusted value is required.  ' +
+            'See http://docs.angularjs.org/api/ng.$sce');
+            expect(function() {
+          $interpolate('{{foo}}{{bar}}', true, isTrustedContext)('val');
         }).toThrowMinErr(
-            '$interpolate', 'noconcat', 'Error while interpolating: {{foo}}{{bar}}\nStrict ' +
-            'Contextual Escaping disallows interpolations that concatenate multiple expressions ' +
-            'when a trusted value is required.  See http://docs.angularjs.org/api/ng.$sce');
+            '$interpolate', 'interr',
+              'Can\'t interpolate: {{foo}}{{bar}}\nError: [$interpolate:noconcat] Error while ' +
+              'interpolating: {{foo}}{{bar}}\nStrict Contextual Escaping disallows interpolations ' +
+              'that concatenate multiple expressions when a trusted value is required.  ' +
+              'See http://docs.angularjs.org/api/ng.$sce');
     }));
 
     it('should interpolate a multi-part expression when isTrustedContext is false', inject(function($interpolate) {
@@ -368,6 +418,23 @@ describe('$interpolate', function() {
       expect($interpolate('some/{{id}}')({id: 1})).toEqual('some/1');
       expect($interpolate('{{foo}}{{bar}}')({foo: 1, bar: 2})).toEqual('12');
     }));
+
+
+    it('should interpolate a multi-part expression when isTrustedContext is URL', inject(function($sce, $interpolate) {
+      expect($interpolate('some/{{id}}', true, $sce.URL)({})).toEqual('some/');
+      expect($interpolate('some/{{id}}', true, $sce.URL)({id: 1})).toEqual('some/1');
+      expect($interpolate('{{foo}}{{bar}}', true, $sce.URL)({foo: 1, bar: 2})).toEqual('12');
+    }));
+
+
+    it('should interpolate and sanitize a multi-part expression when isTrustedContext is URL', inject(function($sce, $interpolate) {
+      expect($interpolate('some/{{id}}', true, $sce.URL)({})).toEqual('some/');
+      expect($interpolate('some/{{id}}', true, $sce.URL)({id: 'javascript:'})).toEqual('some/javascript:');
+      expect($interpolate('{{foo}}{{bar}}', true, $sce.URL)({foo: 'javascript:', bar: 'javascript:'})).toEqual('unsafe:javascript:javascript:');
+    }));
+
+
+
   });
 
 
