@@ -1,5 +1,7 @@
 'use strict';
 
+/* globals nodeBlackList false */
+
 describe('$aria', function() {
   var scope, $compile, element;
 
@@ -9,17 +11,236 @@ describe('$aria', function() {
     dealoc(element);
   });
 
-  function injectScopeAndCompiler() {
-    return inject(function(_$compile_, _$rootScope_) {
-      $compile = _$compile_;
-      scope = _$rootScope_;
-    });
-  }
+  describe('with `ngAriaDisable`', function() {
+    beforeEach(injectScopeAndCompiler);
+    beforeEach(function() {
+      jasmine.addMatchers({
+        toHaveAttribute: function toHaveAttributeMatcher() {
+          return {
+            compare: function toHaveAttributeCompare(element, attr) {
+              var node = element[0];
+              var pass = node.hasAttribute(attr);
+              var message = 'Expected `' + node.outerHTML + '` ' + (pass ? 'not ' : '') +
+                            'to have attribute `' + attr + '`.';
 
-  function compileElement(inputHtml) {
-    element = $compile(inputHtml)(scope);
-    scope.$digest();
-  }
+              return {
+                pass: pass,
+                message: message
+              };
+            }
+          };
+        }
+      });
+    });
+
+    // ariaChecked
+    it('should not attach aria-checked to custom checkbox', function() {
+      compileElement('<div role="checkbox" ng-model="val" ng-aria-disable></div>');
+
+      scope.$apply('val = false');
+      expect(element).not.toHaveAttribute('aria-checked');
+
+      scope.$apply('val = true');
+      expect(element).not.toHaveAttribute('aria-checked');
+    });
+
+    it('should not attach aria-checked to custom radio controls', function() {
+      compileElement(
+          '<div role="radio" ng-model="val" value="one" ng-aria-disable></div>' +
+          '<div role="radio" ng-model="val" value="two" ng-aria-disable></div>');
+
+      var radio1 = element.eq(0);
+      var radio2 = element.eq(1);
+
+      scope.$apply('val = "one"');
+      expect(radio1).not.toHaveAttribute('aria-checked');
+      expect(radio2).not.toHaveAttribute('aria-checked');
+
+      scope.$apply('val = "two"');
+      expect(radio1).not.toHaveAttribute('aria-checked');
+      expect(radio2).not.toHaveAttribute('aria-checked');
+    });
+
+    // ariaDisabled
+    it('should not attach aria-disabled to custom controls', function() {
+      compileElement('<div ng-disabled="val" ng-aria-disable></div>');
+
+      scope.$apply('val = false');
+      expect(element).not.toHaveAttribute('aria-disabled');
+
+      scope.$apply('val = true');
+      expect(element).not.toHaveAttribute('aria-disabled');
+    });
+
+    // ariaHidden
+    it('should not attach aria-hidden to `ngShow`', function() {
+      compileElement('<div ng-show="val" ng-aria-disable></div>');
+
+      scope.$apply('val = false');
+      expect(element).not.toHaveAttribute('aria-hidden');
+
+      scope.$apply('val = true');
+      expect(element).not.toHaveAttribute('aria-hidden');
+    });
+
+    it('should not attach aria-hidden to `ngHide`', function() {
+      compileElement('<div ng-hide="val" ng-aria-disable></div>');
+
+      scope.$apply('val = false');
+      expect(element).not.toHaveAttribute('aria-hidden');
+
+      scope.$apply('val = true');
+      expect(element).not.toHaveAttribute('aria-hidden');
+    });
+
+    // ariaInvalid
+    it('should not attach aria-invalid to input', function() {
+      compileElement('<input ng-model="val" ng-minlength="10" ng-aria-disable />');
+
+      scope.$apply('val = "lt 10"');
+      expect(element).not.toHaveAttribute('aria-invalid');
+
+      scope.$apply('val = "gt 10 characters"');
+      expect(element).not.toHaveAttribute('aria-invalid');
+    });
+
+    it('should not attach aria-invalid to custom controls', function() {
+      compileElement('<div role="textbox" ng-model="val" ng-minlength="10" ng-aria-disable></div>');
+
+      scope.$apply('val = "lt 10"');
+      expect(element).not.toHaveAttribute('aria-invalid');
+
+      scope.$apply('val = "gt 10 characters"');
+      expect(element).not.toHaveAttribute('aria-invalid');
+    });
+
+    // ariaLive
+    it('should not attach aria-live to `ngMessages`', function() {
+      compileElement('<div ng-messages="val" ng-aria-disable>');
+      expect(element).not.toHaveAttribute('aria-live');
+    });
+
+    // ariaReadonly
+    it('should not attach aria-readonly to custom controls', function() {
+      compileElement('<div ng-readonly="val" ng-aria-disable></div>');
+
+      scope.$apply('val = false');
+      expect(element).not.toHaveAttribute('aria-readonly');
+
+      scope.$apply('val = true');
+      expect(element).not.toHaveAttribute('aria-readonly');
+    });
+
+    // ariaRequired
+    it('should not attach aria-required to custom controls with `required`', function() {
+      compileElement('<div ng-model="val" required ng-aria-disable></div>');
+      expect(element).not.toHaveAttribute('aria-required');
+    });
+
+    it('should not attach aria-required to custom controls with `ngRequired`', function() {
+      compileElement('<div ng-model="val" ng-required="val" ng-aria-disable></div>');
+
+      scope.$apply('val = false');
+      expect(element).not.toHaveAttribute('aria-required');
+
+      scope.$apply('val = true');
+      expect(element).not.toHaveAttribute('aria-required');
+    });
+
+    // ariaValue
+    it('should not attach aria-value* to input[range]', function() {
+      compileElement('<input type="range" ng-model="val" min="0" max="100" ng-aria-disable />');
+
+      expect(element).not.toHaveAttribute('aria-valuemax');
+      expect(element).not.toHaveAttribute('aria-valuemin');
+      expect(element).not.toHaveAttribute('aria-valuenow');
+
+      scope.$apply('val = 50');
+      expect(element).not.toHaveAttribute('aria-valuemax');
+      expect(element).not.toHaveAttribute('aria-valuemin');
+      expect(element).not.toHaveAttribute('aria-valuenow');
+
+      scope.$apply('val = 150');
+      expect(element).not.toHaveAttribute('aria-valuemax');
+      expect(element).not.toHaveAttribute('aria-valuemin');
+      expect(element).not.toHaveAttribute('aria-valuenow');
+    });
+
+    it('should not attach aria-value* to custom controls', function() {
+      compileElement(
+          '<div role="progressbar" ng-model="val" min="0" max="100" ng-aria-disable></div>' +
+          '<div role="slider" ng-model="val" min="0" max="100" ng-aria-disable></div>');
+
+      var progressbar = element.eq(0);
+      var slider = element.eq(1);
+
+      ['aria-valuemax', 'aria-valuemin', 'aria-valuenow'].forEach(function(attr) {
+        expect(progressbar).not.toHaveAttribute(attr);
+        expect(slider).not.toHaveAttribute(attr);
+      });
+
+      scope.$apply('val = 50');
+      ['aria-valuemax', 'aria-valuemin', 'aria-valuenow'].forEach(function(attr) {
+        expect(progressbar).not.toHaveAttribute(attr);
+        expect(slider).not.toHaveAttribute(attr);
+      });
+
+      scope.$apply('val = 150');
+      ['aria-valuemax', 'aria-valuemin', 'aria-valuenow'].forEach(function(attr) {
+        expect(progressbar).not.toHaveAttribute(attr);
+        expect(slider).not.toHaveAttribute(attr);
+      });
+    });
+
+    // bindKeypress
+    it('should not bind keypress to `ngClick`', function() {
+      scope.onClick = jasmine.createSpy('onClick');
+      compileElement(
+          '<div ng-click="onClick()" tabindex="0" ng-aria-disable></div>' +
+          '<ul><li ng-click="onClick()" tabindex="0" ng-aria-disable></li></ul>');
+
+      var div = element.find('div');
+      var li = element.find('li');
+
+      div.triggerHandler({type: 'keypress', keyCode: 32});
+      li.triggerHandler({type: 'keypress', keyCode: 32});
+
+      expect(scope.onClick).not.toHaveBeenCalled();
+    });
+
+    // bindRoleForClick
+    it('should not attach role to custom controls', function() {
+      compileElement(
+          '<div ng-click="onClick()" ng-aria-disable></div>' +
+          '<div type="checkbox" ng-model="val" ng-aria-disable></div>' +
+          '<div type="radio" ng-model="val" ng-aria-disable></div>' +
+          '<div type="range" ng-model="val" ng-aria-disable></div>');
+
+      expect(element.eq(0)).not.toHaveAttribute('role');
+      expect(element.eq(1)).not.toHaveAttribute('role');
+      expect(element.eq(2)).not.toHaveAttribute('role');
+      expect(element.eq(3)).not.toHaveAttribute('role');
+    });
+
+    // tabindex
+    it('should not attach tabindex to custom controls', function() {
+      compileElement(
+          '<div role="checkbox" ng-model="val" ng-aria-disable></div>' +
+          '<div role="slider" ng-model="val" ng-aria-disable></div>');
+
+      expect(element.eq(0)).not.toHaveAttribute('tabindex');
+      expect(element.eq(1)).not.toHaveAttribute('tabindex');
+    });
+
+    it('should not attach tabindex to `ngClick` or `ngDblclick`', function() {
+      compileElement(
+          '<div ng-click="onClick()" ng-aria-disable></div>' +
+          '<div ng-dblclick="onDblclick()" ng-aria-disable></div>');
+
+      expect(element.eq(0)).not.toHaveAttribute('tabindex');
+      expect(element.eq(1)).not.toHaveAttribute('tabindex');
+    });
+  });
 
   describe('aria-hidden', function() {
     beforeEach(injectScopeAndCompiler);
@@ -84,7 +305,6 @@ describe('$aria', function() {
     });
   });
 
-
   describe('aria-hidden when disabled', function() {
     beforeEach(configAriaProvider({
       ariaHidden: false
@@ -115,15 +335,41 @@ describe('$aria', function() {
     });
 
     it('should attach itself to custom checkbox', function() {
-      compileElement('<div role="checkbox" ng-model="val">');
+      compileElement('<div role="checkbox" ng-model="val"></div>');
 
-      scope.$apply('val = true');
+      scope.$apply('val = "checked"');
       expect(element.attr('aria-checked')).toBe('true');
 
-      scope.$apply('val = false');
+      scope.$apply('val = null');
       expect(element.attr('aria-checked')).toBe('false');
     });
 
+    it('should use `$isEmpty()` to determine if the checkbox is checked',
+      function() {
+        compileElement('<div role="checkbox" ng-model="val"></div>');
+        var ctrl = element.controller('ngModel');
+        ctrl.$isEmpty = function(value) {
+          return value === 'not-checked';
+        };
+
+        scope.$apply('val = true');
+        expect(ctrl.$modelValue).toBe(true);
+        expect(element.attr('aria-checked')).toBe('true');
+
+        scope.$apply('val = false');
+        expect(ctrl.$modelValue).toBe(false);
+        expect(element.attr('aria-checked')).toBe('true');
+
+        scope.$apply('val = "not-checked"');
+        expect(ctrl.$modelValue).toBe('not-checked');
+        expect(element.attr('aria-checked')).toBe('false');
+
+        scope.$apply('val = "checked"');
+        expect(ctrl.$modelValue).toBe('checked');
+        expect(element.attr('aria-checked')).toBe('true');
+      }
+    );
+
     it('should not handle native checkbox with ngChecked', function() {
       var element = $compile('<input type="checkbox" ng-checked="val">')(scope);
 
@@ -210,11 +456,12 @@ describe('$aria', function() {
     });
 
     it('should attach itself to role="menuitemcheckbox"', function() {
-      scope.val = true;
       compileElement('<div role="menuitemcheckbox" ng-model="val"></div>');
+
+      scope.$apply('val = "checked"');
       expect(element.attr('aria-checked')).toBe('true');
 
-      scope.$apply('val = false');
+      scope.$apply('val = null');
       expect(element.attr('aria-checked')).toBe('false');
     });
 
@@ -272,6 +519,19 @@ describe('$aria', function() {
       compileElement('<input type="range" ng-model="val"/>');
       expect(element.attr('role')).toBeUndefined();
     });
+
+    they('should not add role to native $prop controls', {
+      input: '<input type="text" ng-model="val">',
+      select: '<select type="checkbox" ng-model="val"></select>',
+      textarea: '<textarea type="checkbox" ng-model="val"></textarea>',
+      button: '<button ng-click="doClick()"></button>',
+      summary: '<summary ng-click="doClick()"></summary>',
+      details: '<details ng-click="doClick()"></details>',
+      a: '<a ng-click="doClick()"></a>'
+    }, function(tmpl) {
+      var element = $compile(tmpl)(scope);
+      expect(element.attr('role')).toBeUndefined();
+    });
   });
 
   describe('aria-checked when disabled', function() {
@@ -381,6 +641,21 @@ describe('$aria', function() {
       scope.$apply('txtInput=\'LTten\'');
       expect(element.attr('aria-invalid')).toBe('userSetValue');
     });
+
+    it('should not attach if input is type="hidden"', function() {
+      compileElement('<input type="hidden" ng-model="txtInput">');
+      expect(element.attr('aria-invalid')).toBeUndefined();
+    });
+
+
+    it('should attach aria-invalid to custom control that is type="hidden"', function() {
+      compileElement('<div ng-model="txtInput" type="hidden" role="textbox" ng-minlength="10"></div>');
+      scope.$apply('txtInput=\'LTten\'');
+      expect(element.attr('aria-invalid')).toBe('true');
+
+      scope.$apply('txtInput=\'morethantencharacters\'');
+      expect(element.attr('aria-invalid')).toBe('false');
+    });
   });
 
   describe('aria-invalid when disabled', function() {
@@ -649,90 +924,216 @@ describe('$aria', function() {
   });
 
   describe('accessible actions', function() {
+    var clickEvents;
+
     beforeEach(injectScopeAndCompiler);
-
-    var clickFn;
-
-    it('should trigger a click from the keyboard', function() {
-      scope.someAction = function() {};
-
-      var elements = $compile('<section>' +
-                  '<div class="div-click" ng-click="someAction(\'div\')" tabindex="0"></div>' +
-                  '<ul><li ng-click="someAction( \'li\')" tabindex="0"></li></ul>' +
-                  '</section>')(scope);
-
-      scope.$digest();
-
-      clickFn = spyOn(scope, 'someAction');
-
-      var divElement = elements.find('div');
-      var liElement = elements.find('li');
-
-      divElement.triggerHandler({type: 'keypress', keyCode: 32});
-      liElement.triggerHandler({type: 'keypress', keyCode: 32});
-
-      expect(clickFn).toHaveBeenCalledWith('div');
-      expect(clickFn).toHaveBeenCalledWith('li');
+    beforeEach(function() {
+      clickEvents = [];
+      scope.onClick = jasmine.createSpy('onClick').and.callFake(function(evt) {
+        var nodeName = evt ? evt.target.nodeName.toLowerCase() : '';
+        var prevented = !!(evt && evt.isDefaultPrevented());
+        clickEvents.push(nodeName + '(' + prevented + ')');
+      });
     });
 
-    it('should trigger a click in browsers that provide event.which instead of event.keyCode', function() {
-      scope.someAction = function() {};
+    it('should trigger a click from the keyboard (and prevent default action)', function() {
+      compileElement(
+        '<section>' +
+          '<div ng-click="onClick($event)"></div>' +
+          '<ul><li ng-click="onClick($event)"></li></ul>' +
+        '</section>');
 
-      var elements = $compile('<section>' +
-      '<div class="div-click" ng-click="someAction(\'div\')" tabindex="0"></div>' +
-      '<ul><li ng-click="someAction( \'li\')" tabindex="0"></li></ul>' +
-      '</section>')(scope);
+      var divElement = element.find('div');
+      var liElement = element.find('li');
 
-      scope.$digest();
+      divElement.triggerHandler({type: 'keydown', keyCode: 13});
+      liElement.triggerHandler({type: 'keydown', keyCode: 13});
+      divElement.triggerHandler({type: 'keydown', keyCode: 32});
+      liElement.triggerHandler({type: 'keydown', keyCode: 32});
 
-      clickFn = spyOn(scope, 'someAction');
-
-      var divElement = elements.find('div');
-      var liElement = elements.find('li');
-
-      divElement.triggerHandler({type: 'keypress', which: 32});
-      liElement.triggerHandler({type: 'keypress', which: 32});
-
-      expect(clickFn).toHaveBeenCalledWith('div');
-      expect(clickFn).toHaveBeenCalledWith('li');
+      expect(clickEvents).toEqual(['div(true)', 'li(true)', 'div(true)', 'li(true)']);
     });
 
-    it('should not override existing ng-keypress', function() {
-      scope.someOtherAction = function() {};
-      var keypressFn = spyOn(scope, 'someOtherAction');
+    it('should trigger a click in browsers that provide `event.which` instead of `event.keyCode`',
+      function() {
+        compileElement(
+          '<section>' +
+            '<div ng-click="onClick($event)"></div>' +
+            '<ul><li ng-click="onClick($event)"></li></ul>' +
+          '</section>');
 
-      scope.someAction = function() {};
-      clickFn = spyOn(scope, 'someAction');
-      compileElement('<div ng-click="someAction()" ng-keypress="someOtherAction()" tabindex="0"></div>');
+        var divElement = element.find('div');
+        var liElement = element.find('li');
 
-      element.triggerHandler({type: 'keypress', keyCode: 32});
+        divElement.triggerHandler({type: 'keydown', which: 13});
+        liElement.triggerHandler({type: 'keydown', which: 13});
+        divElement.triggerHandler({type: 'keydown', which: 32});
+        liElement.triggerHandler({type: 'keydown', which: 32});
 
-      expect(clickFn).not.toHaveBeenCalled();
-      expect(keypressFn).toHaveBeenCalled();
+        expect(clickEvents).toEqual(['div(true)', 'li(true)', 'div(true)', 'li(true)']);
+      }
+    );
+
+    it('should not prevent default keyboard action if the target element has editable content',
+      inject(function($document) {
+        // Note:
+        // `contenteditable` is an enumarated (not a boolean) attribute (see
+        // https://developer.mozilla.org/en-US/docs/Web/HTML/Global_attributes/contenteditable).
+        // We need to check the following conditions:
+        //   - No attribute.
+        //   - Value: ""
+        //   - Value: "true"
+        //   - Value: "false"
+
+        function eventFor(keyCode) {
+          return {bubbles: true, cancelable: true, keyCode: keyCode};
+        }
+
+        compileElement(
+          '<section>' +
+            // No attribute.
+            '<div id="no-attribute">' +
+              '<div ng-click="onClick($event)"></div>' +
+              '<ul ng-click="onClick($event)"><li></li></ul>' +
+            '</div>' +
+
+            // Value: ""
+            '<div id="value-empty">' +
+              '<div ng-click="onClick($event)" contenteditable=""></div>' +
+              '<ul ng-click="onClick($event)"><li contenteditable=""></li></ul>' +
+            '</div>' +
+
+            // Value: "true"
+            '<div id="value-true">' +
+              '<div ng-click="onClick($event)" contenteditable="true"></div>' +
+              '<ul ng-click="onClick($event)"><li contenteditable="true"></li></ul>' +
+            '</div>' +
+
+            // Value: "false"
+            '<div id="value-false">' +
+              '<div ng-click="onClick($event)" contenteditable="false"></div>' +
+              '<ul ng-click="onClick($event)"><li contenteditable="false"></li></ul>' +
+            '</div>' +
+          '</section>');
+
+        // Support: Safari 11-12+
+        // Attach to DOM, because otherwise Safari will not update the `isContentEditable` property
+        // based on the `contenteditable` attribute.
+        $document.find('body').append(element);
+
+        var containers = element.children();
+        var container;
+
+        // Using `browserTrigger()`, because it supports event bubbling.
+
+        // No attribute | Elements are not editable.
+        container = containers.eq(0);
+        browserTrigger(container.find('div'), 'keydown', eventFor(13));
+        browserTrigger(container.find('ul'), 'keydown', eventFor(32));
+        browserTrigger(container.find('li'), 'keydown', eventFor(13));
+
+        expect(clickEvents).toEqual(['div(true)', 'ul(true)', 'li(true)']);
+
+        // Value: "" | Elements are editable.
+        clickEvents = [];
+        container = containers.eq(1);
+        browserTrigger(container.find('div'), 'keydown', eventFor(32));
+        browserTrigger(container.find('ul'), 'keydown', eventFor(13));
+        browserTrigger(container.find('li'), 'keydown', eventFor(32));
+
+        expect(clickEvents).toEqual(['div(false)', 'ul(true)', 'li(false)']);
+
+        // Value: "true" | Elements are editable.
+        clickEvents = [];
+        container = containers.eq(2);
+        browserTrigger(container.find('div'), 'keydown', eventFor(13));
+        browserTrigger(container.find('ul'), 'keydown', eventFor(32));
+        browserTrigger(container.find('li'), 'keydown', eventFor(13));
+
+        expect(clickEvents).toEqual(['div(false)', 'ul(true)', 'li(false)']);
+
+        // Value: "false" | Elements are not editable.
+        clickEvents = [];
+        container = containers.eq(3);
+        browserTrigger(container.find('div'), 'keydown', eventFor(32));
+        browserTrigger(container.find('ul'), 'keydown', eventFor(13));
+        browserTrigger(container.find('li'), 'keydown', eventFor(32));
+
+        expect(clickEvents).toEqual(['div(true)', 'ul(true)', 'li(true)']);
+      })
+    );
+
+    they('should not prevent default keyboard action if an interactive $type element' +
+      'is nested inside ng-click', nodeBlackList, function(elementType) {
+        function createHTML(type) {
+          return '<' + type + '></' + type + '>';
+        }
+
+        compileElement(
+          '<section>' +
+            '<div ng-click="onClick($event)">' + createHTML(elementType) + '</div>' +
+          '</section>');
+
+        var divElement = element.find('div');
+        var interactiveElement = element.find(elementType);
+
+        // Use browserTrigger because it supports event bubbling
+        // 13 Enter
+        browserTrigger(interactiveElement, 'keydown', {cancelable: true, bubbles: true, keyCode: 13});
+        expect(clickEvents).toEqual([elementType.toLowerCase() + '(false)']);
+
+        clickEvents = [];
+
+        // 32 Space
+        browserTrigger(interactiveElement, 'keydown', {cancelable: true, bubbles: true, keyCode: 32});
+        expect(clickEvents).toEqual([elementType.toLowerCase() + '(false)']);
+      }
+    );
+
+    they('should not bind to key events if there is existing `ng-$prop`',
+      ['keydown', 'keypress', 'keyup'], function(eventName) {
+        scope.onKeyEvent = jasmine.createSpy('onKeyEvent');
+        compileElement('<div ng-click="onClick()" ng-' + eventName + '="onKeyEvent()"></div>');
+
+        element.triggerHandler({type: eventName, keyCode: 13});
+        element.triggerHandler({type: eventName, keyCode: 32});
+
+        expect(scope.onClick).not.toHaveBeenCalled();
+        expect(scope.onKeyEvent).toHaveBeenCalledTimes(2);
+      }
+    );
+
+    it('should update bindings when keydown is handled', function() {
+      scope.count = 0;
+      compileElement('<div ng-click="count = count + 1">Count: {{ count }}</div>');
+
+      expect(element.text()).toBe('Count: 0');
+
+      element.triggerHandler({type: 'keydown', keyCode: 13});
+      expect(element.text()).toBe('Count: 1');
+
+      element.triggerHandler({type: 'keydown', keyCode: 32});
+      expect(element.text()).toBe('Count: 2');
     });
 
-    it('should update bindings when keypress handled', function() {
-      compileElement('<div ng-click="text = \'clicked!\'">{{text}}</div>');
+    it('should pass `$event` to `ng-click` handler as local', function() {
+      compileElement('<div ng-click="event = $event">{{ event.type }}{{ event.keyCode }}</div>');
       expect(element.text()).toBe('');
-      spyOn(scope.$root, '$digest').and.callThrough();
-      element.triggerHandler({ type: 'keypress', keyCode: 13 });
-      expect(element.text()).toBe('clicked!');
-      expect(scope.$root.$digest).toHaveBeenCalledOnce();
+
+      element.triggerHandler({type: 'keydown', keyCode: 13});
+      expect(element.text()).toBe('keydown13');
+
+      element.triggerHandler({type: 'keydown', keyCode: 32});
+      expect(element.text()).toBe('keydown32');
     });
 
-    it('should pass $event to ng-click handler as local', function() {
-      compileElement('<div ng-click="event = $event">{{event.type}}' +
-                      '{{event.keyCode}}</div>');
-      expect(element.text()).toBe('');
-      element.triggerHandler({ type: 'keypress', keyCode: 13 });
-      expect(element.text()).toBe('keypress13');
-    });
+    it('should not bind keydown to natively interactive elements', function() {
+      compileElement('<button ng-click="onClick()">Click me</button>');
 
-    it('should not bind keypress to elements not in the default config', function() {
-      compileElement('<button ng-click="event = $event">{{event.type}}{{event.keyCode}}</button>');
-      expect(element.text()).toBe('');
-      element.triggerHandler({ type: 'keypress', keyCode: 13 });
-      expect(element.text()).toBe('');
+      element.triggerHandler({type: 'keydown', keyCode: 13});
+      element.triggerHandler({type: 'keydown', keyCode: 32});
+
+      expect(scope.onClick).not.toHaveBeenCalled();
     });
   });
 
@@ -748,21 +1149,29 @@ describe('$aria', function() {
     });
   });
 
-  describe('actions when bindKeypress is set to false', function() {
+  describe('actions when bindKeydown is set to false', function() {
     beforeEach(configAriaProvider({
-      bindKeypress: false
+      bindKeydown: false
     }));
     beforeEach(injectScopeAndCompiler);
 
-    it('should not a trigger click', function() {
-      scope.someAction = function() {};
-      var clickFn = spyOn(scope, 'someAction');
+    it('should not trigger click', function() {
+      scope.someAction = jasmine.createSpy('someAction');
 
       element = $compile('<div ng-click="someAction()" tabindex="0"></div>')(scope);
 
+      element.triggerHandler({type: 'keydown', keyCode: 13});
+      element.triggerHandler({type: 'keydown', keyCode: 32});
+      element.triggerHandler({type: 'keypress', keyCode: 13});
       element.triggerHandler({type: 'keypress', keyCode: 32});
+      element.triggerHandler({type: 'keyup', keyCode: 13});
+      element.triggerHandler({type: 'keyup', keyCode: 32});
 
-      expect(clickFn).not.toHaveBeenCalled();
+      expect(scope.someAction).not.toHaveBeenCalled();
+
+      element.triggerHandler({type: 'click', keyCode: 32});
+
+      expect(scope.someAction).toHaveBeenCalledOnce();
     });
   });
 
@@ -786,19 +1195,53 @@ describe('$aria', function() {
       expect(element.attr('tabindex')).toBeUndefined();
     });
   });
-});
 
-function expectAriaAttrOnEachElement(elem, ariaAttr, expected) {
-  angular.forEach(elem, function(val) {
-    expect(angular.element(val).attr(ariaAttr)).toBe(expected);
-  });
-}
+  describe('ngModel', function() {
+    it('should not break when manually compiling', function() {
+      module(function($compileProvider) {
+        $compileProvider.directive('foo', function() {
+          return {
+            priority: 10,
+            terminal: true,
+            link: function(scope, elem) {
+              $compile(elem, null, 10)(scope);
+            }
+          };
+        });
+      });
 
-function configAriaProvider(config) {
-  return function() {
-    angular.module('ariaTest', ['ngAria']).config(function($ariaProvider) {
-      $ariaProvider.config(config);
+      injectScopeAndCompiler();
+      compileElement('<div role="checkbox" ng-model="value" foo />');
+
+      // Just check an arbitrary feature to make sure it worked
+      expect(element.attr('tabindex')).toBe('0');
     });
-    module('ariaTest');
-  };
-}
+  });
+
+  // Helpers
+  function compileElement(inputHtml) {
+    element = $compile(inputHtml)(scope);
+    scope.$digest();
+  }
+
+  function configAriaProvider(config) {
+    return function() {
+      module(function($ariaProvider) {
+        $ariaProvider.config(config);
+      });
+    };
+  }
+
+  function expectAriaAttrOnEachElement(elem, ariaAttr, expected) {
+    angular.forEach(elem, function(val) {
+      expect(angular.element(val).attr(ariaAttr)).toBe(expected);
+    });
+  }
+
+  function injectScopeAndCompiler() {
+    return inject(function(_$compile_, _$rootScope_) {
+      $compile = _$compile_;
+      scope = _$rootScope_;
+    });
+  }
+});
