@@ -1,9 +1,11 @@
 'use strict';
 
 describe('resource', function() {
+  var noop = angular.noop;
+  var extend = angular.extend;
 
 describe('basic usage', function() {
-  var $resource, CreditCard, callback, $httpBackend, resourceProvider;
+  var $resource, CreditCard, callback, $httpBackend, resourceProvider, $q;
 
   beforeEach(module('ngResource'));
 
@@ -14,6 +16,7 @@ describe('basic usage', function() {
   beforeEach(inject(function($injector) {
     $httpBackend = $injector.get('$httpBackend');
     $resource = $injector.get('$resource');
+    $q = $injector.get('$q');
     CreditCard = $resource('/CreditCard/:id:verb', {id:'@id.key'}, {
       charge:{
         method:'post',
@@ -33,11 +36,11 @@ describe('basic usage', function() {
     callback = jasmine.createSpy('callback');
   }));
 
-
   afterEach(function() {
     $httpBackend.verifyNoOutstandingExpectation();
   });
 
+
   describe('isValidDottedPath', function() {
     /* global isValidDottedPath: false */
     it('should support arbitrary dotted names', function() {
@@ -97,6 +100,76 @@ describe('basic usage', function() {
     $httpBackend.flush();
   });
 
+  it('should include a request body when calling custom method with hasBody is true', function() {
+    var instant = {name: 'info.txt'};
+    var condition = {at: '2038-01-19 03:14:08'};
+
+    $httpBackend.expect('CREATE', '/fooresource', instant).respond({fid: 42});
+    $httpBackend.expect('DELETE', '/fooresource', condition).respond({});
+
+    var r = $resource('/fooresource', {}, {
+      create: {method: 'CREATE', hasBody: true},
+      delete: {method: 'DELETE', hasBody: true}
+    });
+
+    var creationResponse = r.create(instant);
+    var deleteResponse = r.delete(condition);
+
+    $httpBackend.flush();
+
+    expect(creationResponse.fid).toBe(42);
+    expect(deleteResponse.$resolved).toBe(true);
+  });
+
+  it('should not include a request body if hasBody is false on POST, PUT and PATCH', function() {
+    function verifyRequest(method, url, data) {
+      expect(data).toBeUndefined();
+      return [200, {id: 42}];
+    }
+
+    $httpBackend.expect('POST', '/foo').respond(verifyRequest);
+    $httpBackend.expect('PUT', '/foo').respond(verifyRequest);
+    $httpBackend.expect('PATCH', '/foo').respond(verifyRequest);
+
+    var R = $resource('/foo', {}, {
+      post: {method: 'POST', hasBody: false},
+      put: {method: 'PUT', hasBody: false},
+      patch: {method: 'PATCH', hasBody: false}
+    });
+
+    var postResponse = R.post();
+    var putResponse = R.put();
+    var patchResponse = R.patch();
+
+    $httpBackend.flush();
+
+    expect(postResponse.id).toBe(42);
+    expect(putResponse.id).toBe(42);
+    expect(patchResponse.id).toBe(42);
+  });
+
+  it('should expect a body if hasBody is true', function() {
+    var username = 'yathos';
+    var loginRequest = {name: username, password: 'Smile'};
+    var user = {id: 1, name: username};
+
+    $httpBackend.expect('LOGIN', '/user/me', loginRequest).respond(user);
+
+    $httpBackend.expect('LOGOUT', '/user/me', null).respond(null);
+
+    var UserService = $resource('/user/me', {}, {
+      login: {method: 'LOGIN', hasBody: true},
+      logout: {method: 'LOGOUT', hasBody: false}
+    });
+
+    var loginResponse = UserService.login(loginRequest);
+    var logoutResponse = UserService.logout();
+
+    $httpBackend.flush();
+
+    expect(loginResponse.id).toBe(user.id);
+    expect(logoutResponse.$resolved).toBe(true);
+  });
 
   it('should build resource', function() {
     expect(typeof CreditCard).toBe('function');
@@ -152,6 +225,23 @@ describe('basic usage', function() {
   });
 
 
+  it('should not throw if response.data is the resource object', function() {
+    var data = {id:{key:123}, number:'9876'};
+    $httpBackend.expect('GET', '/CreditCard/123').respond(data);
+
+    var cc = CreditCard.get({id:123});
+    $httpBackend.flush();
+    expect(cc instanceof CreditCard).toBe(true);
+
+    $httpBackend.expect('POST', '/CreditCard/123', angular.toJson(data)).respond(cc);
+
+    cc.$save();
+    $httpBackend.flush();
+    expect(cc.id).toEqual({key:123});
+    expect(cc.number).toEqual('9876');
+  });
+
+
   it('should default to empty parameters', function() {
     $httpBackend.expect('GET', 'URL').respond({});
     $resource('URL').query();
@@ -237,10 +327,14 @@ describe('basic usage', function() {
     $httpBackend.expect('GET', '/Path/foo%231').respond('{}');
     $httpBackend.expect('GET', '/Path/doh!@foo?bar=baz%231').respond('{}');
     $httpBackend.expect('GET', '/Path/herp$').respond('{}');
+    $httpBackend.expect('GET', '/Path/foo;bar').respond('{}');
+    $httpBackend.expect('GET', '/Path/foo?bar=baz;qux').respond('{}');
 
     R.get({a: 'foo#1'});
     R.get({a: 'doh!@foo', bar: 'baz#1'});
     R.get({a: 'herp$'});
+    R.get({a: 'foo;bar'});
+    R.get({a: 'foo', bar: 'baz;qux'});
   });
 
   it('should not encode @ in url params', function() {
@@ -300,11 +394,35 @@ describe('basic usage', function() {
   });
 
   it('should support IPv6 URLs', function() {
-    var R = $resource('http://[2620:0:861:ed1a::1]/:ed1a/', {}, {}, {stripTrailingSlashes: false});
-    $httpBackend.expect('GET', 'http://[2620:0:861:ed1a::1]/foo/').respond({});
-    $httpBackend.expect('GET', 'http://[2620:0:861:ed1a::1]/').respond({});
-    R.get({ed1a: 'foo'});
-    R.get({});
+    test('http://[2620:0:861:ed1a::1]',        {ed1a: 'foo'}, 'http://[2620:0:861:ed1a::1]');
+    test('http://[2620:0:861:ed1a::1]/',       {ed1a: 'foo'}, 'http://[2620:0:861:ed1a::1]/');
+    test('http://[2620:0:861:ed1a::1]/:ed1a',  {ed1a: 'foo'}, 'http://[2620:0:861:ed1a::1]/foo');
+    test('http://[2620:0:861:ed1a::1]/:ed1a',  {},            'http://[2620:0:861:ed1a::1]/');
+    test('http://[2620:0:861:ed1a::1]/:ed1a/', {ed1a: 'foo'}, 'http://[2620:0:861:ed1a::1]/foo/');
+    test('http://[2620:0:861:ed1a::1]/:ed1a/', {},            'http://[2620:0:861:ed1a::1]/');
+
+    // Helpers
+    function test(templateUrl, params, actualUrl) {
+      var R = $resource(templateUrl, null, null, {stripTrailingSlashes: false});
+      $httpBackend.expect('GET', actualUrl).respond(null);
+      R.get(params);
+    }
+  });
+
+  it('should support params in the `hostname` part of the URL', function() {
+    test('http://:hostname',            {hostname: 'foo.com'},              'http://foo.com');
+    test('http://:hostname/',           {hostname: 'foo.com'},              'http://foo.com/');
+    test('http://:l2Domain.:l1Domain',  {l1Domain: 'com', l2Domain: 'bar'}, 'http://bar.com');
+    test('http://:l2Domain.:l1Domain/', {l1Domain: 'com', l2Domain: 'bar'}, 'http://bar.com/');
+    test('http://127.0.0.:octet',       {octet: 42},                        'http://127.0.0.42');
+    test('http://127.0.0.:octet/',      {octet: 42},                        'http://127.0.0.42/');
+
+    // Helpers
+    function test(templateUrl, params, actualUrl) {
+      var R = $resource(templateUrl, null, null, {stripTrailingSlashes: false});
+      $httpBackend.expect('GET', actualUrl).respond(null);
+      R.get(params);
+    }
   });
 
   it('should support overriding provider default trailing-slash stripping configuration', function() {
@@ -613,30 +731,6 @@ describe('basic usage', function() {
   });
 
 
-  it('should bind default parameters', function() {
-    $httpBackend.expect('GET', '/CreditCard/123.visa?minimum=0.05').respond({id: 123});
-    var Visa = CreditCard.bind({verb: '.visa', minimum: 0.05});
-    var visa = Visa.get({id: 123});
-    $httpBackend.flush();
-    expect(visa).toEqualData({id: 123});
-  });
-
-
-  it('should pass all original arguments when binding default params', function() {
-    $httpBackend.expect('GET', '/CancellableCreditCard/123.visa').respond({id: 123});
-
-    var CancellableCreditCard = $resource('/CancellableCreditCard/:id:verb', {},
-                                          {charge: {method: 'POST'}}, {cancellable: true});
-    var CancellableVisa = CancellableCreditCard.bind({verb: '.visa'});
-    var visa = CancellableVisa.get({id: 123});
-
-    $httpBackend.flush();
-
-    expect(visa.$charge).toEqual(jasmine.any(Function));
-    expect(visa.$cancelRequest).toEqual(jasmine.any(Function));
-  });
-
-
   it('should support dynamic default parameters (global)', function() {
     var currentGroup = 'students',
         Person = $resource('/Person/:group/:id', { group: function() { return currentGroup; }});
@@ -732,6 +826,24 @@ describe('basic usage', function() {
     expect(json).toEqual({id: 123, number: '9876', $myProp: 'still here'});
   });
 
+  it('should not include $cancelRequest when resource is toJson\'ed', function() {
+    $httpBackend.whenGET('/CreditCard').respond({});
+
+    var CreditCard = $resource('/CreditCard', {}, {
+      get: {
+        method: 'GET',
+        cancellable: true
+      }
+    });
+
+    var card = CreditCard.get();
+    var json = card.toJSON();
+
+    expect(card.$cancelRequest).toBeDefined();
+    expect(json.$cancelRequest).toBeUndefined();
+  });
+
+
   describe('promise api', function() {
 
     var $rootScope;
@@ -1020,101 +1132,407 @@ describe('basic usage', function() {
     });
 
 
-    it('should allow per action response interceptor that gets full response', function() {
-      CreditCard = $resource('/CreditCard', {}, {
-        query: {
-          method: 'get',
-          isArray: true,
-          interceptor: {
-            response: function(response) {
-              return response;
-            }
-          }
-        }
+    describe('requestInterceptor', function() {
+      var rejectReason = {'lol':'cat'};
+      var successSpy, failureSpy;
+
+      beforeEach(function() {
+        successSpy = jasmine.createSpy('successSpy');
+        failureSpy = jasmine.createSpy('failureSpy');
       });
 
-      $httpBackend.expect('GET', '/CreditCard').respond([{id: 1}]);
-
-      var ccs = CreditCard.query();
-
-      ccs.$promise.then(callback);
-
-      $httpBackend.flush();
-      expect(callback).toHaveBeenCalledOnce();
-
-      var response = callback.calls.mostRecent().args[0];
-      expect(response.resource).toBe(ccs);
-      expect(response.status).toBe(200);
-      expect(response.config).toBeDefined();
-    });
-
-
-    it('should allow per action responseError interceptor that gets full response', function() {
-      CreditCard = $resource('/CreditCard', {}, {
-        query: {
-          method: 'get',
-          isArray: true,
-          interceptor: {
-            responseError: function(response) {
-              return response;
+      it('should allow per action request interceptor that gets full configuration', function() {
+        var CreditCard = $resource('/CreditCard', {}, {
+          query: {
+            method: 'get',
+            isArray: true,
+            interceptor: {
+              request: function(httpConfig) {
+                callback(httpConfig);
+                return httpConfig;
+              }
             }
           }
-        }
-      });
-
-      $httpBackend.expect('GET', '/CreditCard').respond(404);
-
-      var ccs = CreditCard.query();
-
-      ccs.$promise.then(callback);
-
-      $httpBackend.flush();
-      expect(callback).toHaveBeenCalledOnce();
-
-      var response = callback.calls.mostRecent().args[0];
-      expect(response.status).toBe(404);
-      expect(response.config).toBeDefined();
-    });
-
-
-    it('should fulfill the promise with the value returned by the responseError interceptor',
-      inject(function($q) {
-        CreditCard = $resource('/CreditCard', {}, {
-          test1: {
-            method: 'GET',
-            interceptor: {responseError: function() { return 'foo'; }}
-          },
-          test2: {
-            method: 'GET',
-            interceptor: {responseError: function() { return $q.resolve('bar'); }}
-          },
-          test3: {
-            method: 'GET',
-            interceptor: {responseError: function() { return $q.reject('baz'); }}
-          }
         });
 
-        $httpBackend.whenGET('/CreditCard').respond(404);
+        $httpBackend.expect('GET', '/CreditCard').respond([{id: 1}]);
 
-        callback.calls.reset();
-        CreditCard.test1().$promise.then(callback);
+        var resource = CreditCard.query();
+        resource.$promise.then(successSpy, failureSpy);
+
+        $httpBackend.flush();
+        expect(callback).toHaveBeenCalledOnce();
+        expect(successSpy).toHaveBeenCalledOnce();
+        expect(failureSpy).not.toHaveBeenCalled();
+
+        expect(callback).toHaveBeenCalledWith({
+          'method': 'get',
+          'url': '/CreditCard'
+        });
+      });
+
+      it('should call $http with the value returned from requestInterceptor', function() {
+        var CreditCard = $resource('/CreditCard', {}, {
+          query: {
+            method: 'get',
+            isArray: true,
+            interceptor: {
+              request: function(httpConfig) {
+                httpConfig.url = '/DebitCard';
+                return httpConfig;
+              }
+            }
+          }
+        });
+
+        $httpBackend.expect('GET', '/DebitCard').respond([{id: 1}]);
+
+        var resource = CreditCard.query();
+        resource.$promise.then(successSpy, failureSpy);
+
+        $httpBackend.flush();
+        expect(successSpy).toHaveBeenCalledOnceWith(jasmine.arrayContaining([
+          jasmine.objectContaining({id: 1})
+        ]));
+        expect(failureSpy).not.toHaveBeenCalled();
+      });
+
+      it('should abort the operation if the requestInterceptor rejects the operation', function() {
+        var CreditCard = $resource('/CreditCard', {}, {
+          query: {
+            method: 'get',
+            isArray: true,
+            interceptor: {
+              request: function() {
+                return $q.reject(rejectReason);
+              }
+            }
+          }
+        });
+
+        var resource = CreditCard.query();
+        resource.$promise.then(successSpy, failureSpy);
+
+        // Make sure all promises resolve.
+        $rootScope.$apply();
+
+        // Ensure the resource promise was rejected
+        expect(resource.$resolved).toBeTruthy();
+        expect(successSpy).not.toHaveBeenCalled();
+        expect(failureSpy).toHaveBeenCalledOnceWith(rejectReason);
+
+        // Ensure that no requests were made.
+        $httpBackend.verifyNoOutstandingRequest();
+      });
+
+      it('should call requestErrorInterceptor if requestInterceptor rejects the operation', function() {
+        var CreditCard = $resource('/CreditCard', {}, {
+          query: {
+            method: 'get',
+            isArray: true,
+            interceptor: {
+              request: function() {
+                return $q.reject(rejectReason);
+              },
+              requestError: function(rejection) {
+                callback(rejection);
+                return $q.reject(rejection);
+              }
+            }
+          }
+        });
+
+        var resource = CreditCard.query();
+        resource.$promise.then(successSpy, failureSpy);
+        $rootScope.$digest();
+
+        expect(callback).toHaveBeenCalledOnceWith(rejectReason);
+        expect(successSpy).not.toHaveBeenCalled();
+        expect(failureSpy).toHaveBeenCalledOnceWith(rejectReason);
+
+        // Ensure that no requests were made.
+        $httpBackend.verifyNoOutstandingRequest();
+      });
+
+      it('should abort the operation if a requestErrorInterceptor rejects the operation', function() {
+        var CreditCard = $resource('/CreditCard', {}, {
+          query: {
+            method: 'get',
+            isArray: true,
+            interceptor: {
+              request: function() {
+                return $q.reject(rejectReason);
+              },
+              requestError: function(rejection) {
+                return $q.reject(rejection);
+              }
+            }
+          }
+        });
+
+        var resource = CreditCard.query();
+        resource.$promise.then(successSpy, failureSpy);
+        $rootScope.$apply();
+
+        expect(resource.$resolved).toBeTruthy();
+        expect(successSpy).not.toHaveBeenCalled();
+        expect(failureSpy).toHaveBeenCalledOnceWith(rejectReason);
+
+        // Ensure that no requests were made.
+        $httpBackend.verifyNoOutstandingRequest();
+      });
+
+      it('should continue the operation if a requestErrorInterceptor rescues it', function() {
+        var CreditCard = $resource('/CreditCard', {}, {
+          query: {
+            method: 'get',
+            isArray: true,
+            interceptor: {
+              request: function(httpConfig) {
+                return $q.reject(httpConfig);
+              },
+              requestError: function(httpConfig) {
+                return $q.resolve(httpConfig);
+              }
+            }
+          }
+        });
+
+        $httpBackend.expect('GET', '/CreditCard').respond([{id: 1}]);
+
+        var resource = CreditCard.query();
+        resource.$promise.then(successSpy, failureSpy);
         $httpBackend.flush();
 
+        expect(resource.$resolved).toBeTruthy();
+        expect(successSpy).toHaveBeenCalledOnceWith(jasmine.arrayContaining([
+          jasmine.objectContaining({id: 1})
+        ]));
+        expect(failureSpy).not.toHaveBeenCalled();
+
+        $httpBackend.verifyNoOutstandingRequest();
+      });
+    });
+
+
+    describe('responseInterceptor', function() {
+      it('should allow per action response interceptor that gets full response', function() {
+        var response;
+
+        $httpBackend.expect('GET', '/CreditCard').respond(201, {id: 1}, {foo: 'bar'}, 'Ack');
+        CreditCard = $resource('/CreditCard', {}, {
+          get: {
+            method: 'get',
+            interceptor: {response: function(resp) { response = resp; }}
+          }
+        });
+
+        var cc = CreditCard.get();
+        $httpBackend.flush();
+
+        expect(response.resource).toBe(cc);
+        expect(response.config).toBeDefined();
+        expect(response.status).toBe(201);
+        expect(response.statusText).toBe('Ack');
+        expect(response.headers()).toEqual({foo: 'bar'});
+      });
+
+
+      it('should allow per action responseError interceptor that gets full response', function() {
+        var response;
+
+        $httpBackend.expect('GET', '/CreditCard').respond(404, {ignored: 'stuff'}, {foo: 'bar'}, 'Ack');
+        CreditCard = $resource('/CreditCard', {}, {
+          get: {
+            method: 'get',
+            interceptor: {responseError: function(resp) { response = resp; }}
+          }
+        });
+
+        var cc = CreditCard.get();
+        $httpBackend.flush();
+
+        expect(response.resource).toBe(cc);
+        expect(response.config).toBeDefined();
+        expect(response.status).toBe(404);
+        expect(response.statusText).toBe('Ack');
+        expect(response.headers()).toEqual({foo: 'bar'});
+      });
+
+
+      it('should fulfill the promise with the value returned by the response interceptor',
+        function() {
+          $httpBackend.whenGET('/CreditCard').respond(200);
+          CreditCard = $resource('/CreditCard', {}, {
+            test1: {
+              method: 'get',
+              interceptor: {response: function() { return 'foo'; }}
+            },
+            test2: {
+              method: 'get',
+              interceptor: {response: function() { return $q.resolve('bar'); }}
+            },
+            test3: {
+              method: 'get',
+              interceptor: {response: function() { return $q.reject('baz'); }}
+            }
+          });
+
+          CreditCard.test1().$promise.then(callback);
+          $httpBackend.flush();
+          expect(callback).toHaveBeenCalledOnceWith('foo');
+
+          callback.calls.reset();
+
+          CreditCard.test2().$promise.then(callback);
+          $httpBackend.flush();
+          expect(callback).toHaveBeenCalledOnceWith('bar');
+
+          callback.calls.reset();
+
+          CreditCard.test3().$promise.then(null, callback);
+          $httpBackend.flush();
+          expect(callback).toHaveBeenCalledOnceWith('baz');
+        }
+      );
+
+
+      it('should fulfill the promise with the value returned by the responseError interceptor',
+        function() {
+          $httpBackend.whenGET('/CreditCard').respond(404);
+          CreditCard = $resource('/CreditCard', {}, {
+            test1: {
+              method: 'get',
+              interceptor: {responseError: function() { return 'foo'; }}
+            },
+            test2: {
+              method: 'get',
+              interceptor: {responseError: function() { return $q.resolve('bar'); }}
+            },
+            test3: {
+              method: 'get',
+              interceptor: {responseError: function() { return $q.reject('baz'); }}
+            }
+          });
+
+          CreditCard.test1().$promise.then(callback);
+          $httpBackend.flush();
+          expect(callback).toHaveBeenCalledOnceWith('foo');
+
+          callback.calls.reset();
+
+          CreditCard.test2().$promise.then(callback);
+          $httpBackend.flush();
+          expect(callback).toHaveBeenCalledOnceWith('bar');
+
+          callback.calls.reset();
+
+          CreditCard.test3().$promise.then(null, callback);
+          $httpBackend.flush();
+          expect(callback).toHaveBeenCalledOnceWith('baz');
+        }
+      );
+
+
+      it('should call the success callback when response interceptor succeeds', function() {
+        $httpBackend.whenGET('/CreditCard').respond(200);
+        CreditCard = $resource('/CreditCard', {}, {
+          test1: {
+            method: 'get',
+            interceptor: {response: function() { return 'foo'; }}
+          },
+          test2: {
+            method: 'get',
+            interceptor: {response: function() { return $q.resolve('bar'); }}
+          }
+        });
+
+        CreditCard.test1(callback);
+        $httpBackend.flush();
+        expect(callback).toHaveBeenCalledOnceWith('foo', jasmine.any(Function), 200, '');
+
+        callback.calls.reset();
+
+        CreditCard.test2(callback);
+        $httpBackend.flush();
+        expect(callback).toHaveBeenCalledOnceWith('bar', jasmine.any(Function), 200, '');
+      });
+
+
+      it('should call the error callback when response interceptor fails', function() {
+        $httpBackend.whenGET('/CreditCard').respond(200);
+        CreditCard = $resource('/CreditCard', {}, {
+          test1: {
+            method: 'get',
+            interceptor: {response: function() { throw 'foo'; }}
+          },
+          test2: {
+            method: 'get',
+            interceptor: {response: function() { return $q.reject('bar'); }}
+          }
+        });
+
+        CreditCard.test1(noop, callback);
+        $httpBackend.flush();
         expect(callback).toHaveBeenCalledOnceWith('foo');
 
         callback.calls.reset();
-        CreditCard.test2().$promise.then(callback);
-        $httpBackend.flush();
 
+        CreditCard.test2(noop, callback);
+        $httpBackend.flush();
         expect(callback).toHaveBeenCalledOnceWith('bar');
+      });
+
+
+      it('should call the success callback when responseError interceptor succeeds', function() {
+        $httpBackend.whenGET('/CreditCard').respond(404);
+        CreditCard = $resource('/CreditCard', {}, {
+          test1: {
+            method: 'get',
+            interceptor: {responseError: function() { return 'foo'; }}
+          },
+          test2: {
+            method: 'get',
+            interceptor: {responseError: function() { return $q.resolve('bar'); }}
+          }
+        });
+
+        CreditCard.test1(callback);
+        $httpBackend.flush();
+        expect(callback).toHaveBeenCalledOnceWith('foo', jasmine.any(Function), 404, '');
 
         callback.calls.reset();
-        CreditCard.test3().$promise.then(null, callback);
-        $httpBackend.flush();
 
-        expect(callback).toHaveBeenCalledOnceWith('baz');
-      })
-    );
+        CreditCard.test2(callback);
+        $httpBackend.flush();
+        expect(callback).toHaveBeenCalledOnceWith('bar', jasmine.any(Function), 404, '');
+      });
+
+
+      it('should call the error callback when responseError interceptor fails', function() {
+        $httpBackend.whenGET('/CreditCard').respond(404);
+        CreditCard = $resource('/CreditCard', {}, {
+          test1: {
+            method: 'get',
+            interceptor: {responseError: function() { throw 'foo'; }}
+          },
+          test2: {
+            method: 'get',
+            interceptor: {responseError: function() { return $q.reject('bar'); }}
+          }
+        });
+
+        CreditCard.test1(noop, callback);
+        $httpBackend.flush();
+        expect(callback).toHaveBeenCalledOnceWith('foo');
+
+        callback.calls.reset();
+
+        CreditCard.test2(noop, callback);
+        $httpBackend.flush();
+        expect(callback).toHaveBeenCalledOnceWith('bar');
+      });
+    });
   });
 
 
@@ -1393,6 +1811,18 @@ describe('basic usage', function() {
         $httpBackend.expect('POST', '/users/.json').respond();
         $resource('/users/\\.json').save({});
       });
+      it('should work with save() if dynamic params', function() {
+        $httpBackend.expect('POST', '/users/.json').respond();
+        $resource('/users/:json', {json: '\\.json'}).save({});
+      });
+      it('should work with query() if dynamic params', function() {
+        $httpBackend.expect('GET', '/users/.json').respond();
+        $resource('/users/:json', {json: '\\.json'}).query();
+      });
+      it('should work with get() if dynamic params', function() {
+        $httpBackend.expect('GET', '/users/.json').respond();
+        $resource('/users/:json', {json: '\\.json'}).get();
+      });
     });
   });
 
@@ -1458,8 +1888,54 @@ describe('basic usage', function() {
   });
 });
 
+describe('extra params', function() {
+  var $http;
+  var $httpBackend;
+  var $resource;
+  var $rootScope;
+
+  beforeEach(module('ngResource'));
+
+  beforeEach(module(function($provide) {
+    $provide.decorator('$http', function($delegate) {
+      return jasmine.createSpy('$http').and.callFake($delegate);
+    });
+  }));
+
+  beforeEach(inject(function(_$http_, _$httpBackend_, _$resource_, _$rootScope_) {
+    $http = _$http_;
+    $httpBackend = _$httpBackend_;
+    $resource = _$resource_;
+    $rootScope = _$rootScope_;
+  }));
+
+  afterEach(function() {
+    $httpBackend.verifyNoOutstandingExpectation();
+  });
+
+
+  it('should pass extra params to `$http` as `config.params`', function() {
+    $httpBackend.expectGET('/bar?baz=qux').respond('{}');
+
+    var R = $resource('/:foo');
+    R.get({foo: 'bar', baz: 'qux'});
+
+    $rootScope.$digest();
+    expect($http).toHaveBeenCalledWith(jasmine.objectContaining({params: {baz: 'qux'}}));
+  });
+
+  it('should pass extra params even if `Object.prototype` has properties with the same name',
+    function() {
+      $httpBackend.expectGET('/foo?toString=bar').respond('{}');
+
+      var R = $resource('/foo');
+      R.get({toString: 'bar'});
+    }
+  );
+});
+
 describe('errors', function() {
-  var $httpBackend, $resource, $q;
+  var $httpBackend, $resource;
 
   beforeEach(module(function($exceptionHandlerProvider) {
     $exceptionHandlerProvider.mode('log');
@@ -1470,7 +1946,6 @@ describe('errors', function() {
   beforeEach(inject(function($injector) {
     $httpBackend = $injector.get('$httpBackend');
     $resource = $injector.get('$resource');
-    $q = $injector.get('$q');
   }));
 
 
@@ -1510,12 +1985,17 @@ describe('errors', function() {
 });
 
 describe('handling rejections', function() {
+  var $exceptionHandler;
   var $httpBackend;
   var $resource;
 
   beforeEach(module('ngResource'));
+  beforeEach(module(function($exceptionHandlerProvider) {
+    $exceptionHandlerProvider.mode('log');
+  }));
 
-  beforeEach(inject(function(_$httpBackend_, _$resource_) {
+  beforeEach(inject(function(_$exceptionHandler_, _$httpBackend_, _$resource_) {
+    $exceptionHandler = _$exceptionHandler_;
     $httpBackend = _$httpBackend_;
     $resource = _$resource_;
 
@@ -1528,12 +2008,256 @@ describe('handling rejections', function() {
     var errorCb2 = jasmine.createSpy('errorCb2');
     var CreditCard = $resource('/CreditCard');
 
-    CreditCard.get(noop, errorCb1).$promise['catch'](errorCb2);
+    CreditCard.get(noop, errorCb1).$promise.catch(errorCb2);
     $httpBackend.flush();
 
     expect(errorCb1).toHaveBeenCalledOnce();
     expect(errorCb2).toHaveBeenCalledOnce();
   });
+
+
+  it('should report a PUR when no error callback or responseError interceptor is provided',
+    function() {
+      var CreditCard = $resource('/CreditCard');
+
+      CreditCard.get();
+      $httpBackend.flush();
+
+      expect($exceptionHandler.errors.length).toBe(1);
+      expect($exceptionHandler.errors[0]).toMatch(/^Possibly unhandled rejection/);
+    }
+  );
+
+
+  it('should not report a PUR when an error callback or responseError interceptor is provided',
+    function() {
+      var CreditCard = $resource('/CreditCard', {}, {
+        test1: {
+          method: 'GET'
+        },
+        test2: {
+          method: 'GET',
+          interceptor: {responseError: function() { return {}; }}
+        }
+      });
+
+      // With error callback
+      CreditCard.test1(noop, noop);
+      $httpBackend.flush();
+
+      expect($exceptionHandler.errors.length).toBe(0);
+
+      // With responseError interceptor
+      CreditCard.test2();
+      $httpBackend.flush();
+
+      expect($exceptionHandler.errors.length).toBe(0);
+
+      // With error callback and responseError interceptor
+      CreditCard.test2(noop, noop);
+      $httpBackend.flush();
+
+      expect($exceptionHandler.errors.length).toBe(0);
+    }
+  );
+
+
+  it('should report a PUR when the responseError interceptor returns a rejected promise',
+    inject(function($q) {
+      var CreditCard = $resource('/CreditCard', {}, {
+        test: {
+          method: 'GET',
+          interceptor: {responseError: function() { return $q.reject({}); }}
+        }
+      });
+
+      CreditCard.test();
+      $httpBackend.flush();
+
+      expect($exceptionHandler.errors.length).toBe(1);
+      expect($exceptionHandler.errors[0]).toMatch(/^Possibly unhandled rejection/);
+    })
+  );
+
+
+  it('should not swallow exceptions in success callback when error callback is provided',
+    function() {
+      $httpBackend.expectGET('/CreditCard/123').respond(null);
+      var CreditCard = $resource('/CreditCard/:id');
+      CreditCard.get({id: 123},
+          function(res) { throw new Error('should be caught'); },
+          function() {});
+
+      $httpBackend.flush();
+      expect($exceptionHandler.errors.length).toBe(1);
+      expect($exceptionHandler.errors[0]).toMatch(/^Error: should be caught/);
+    }
+  );
+
+
+  it('should not swallow exceptions in success callback when error callback is not provided',
+    function() {
+      $httpBackend.expectGET('/CreditCard/123').respond(null);
+      var CreditCard = $resource('/CreditCard/:id');
+      CreditCard.get({id: 123},
+          function(res) { throw new Error('should be caught'); });
+
+      $httpBackend.flush();
+      expect($exceptionHandler.errors.length).toBe(1);
+      expect($exceptionHandler.errors[0]).toMatch(/^Error: should be caught/);
+    }
+  );
+
+
+  it('should not swallow exceptions in success callback when error callback is provided and has responseError interceptor',
+    function() {
+      $httpBackend.expectGET('/CreditCard/123').respond(null);
+      var CreditCard = $resource('/CreditCard/:id', null, {
+        get: {
+          method: 'GET',
+          interceptor: {responseError: function() {}}
+        }
+      });
+
+      CreditCard.get({id: 123},
+          function(res) { throw new Error('should be caught'); },
+          function() {});
+
+      $httpBackend.flush();
+      expect($exceptionHandler.errors.length).toBe(1);
+      expect($exceptionHandler.errors[0]).toMatch(/^Error: should be caught/);
+    }
+  );
+
+
+  it('should not swallow exceptions in success callback when error callback is not provided and has responseError interceptor',
+    function() {
+      $httpBackend.expectGET('/CreditCard/123').respond(null);
+      var CreditCard = $resource('/CreditCard/:id', null, {
+        get: {
+          method: 'GET',
+          interceptor: {responseError: function() {}}
+        }
+      });
+
+      CreditCard.get({id: 123},
+          function(res) { throw new Error('should be caught'); });
+
+      $httpBackend.flush();
+      expect($exceptionHandler.errors.length).toBe(1);
+      expect($exceptionHandler.errors[0]).toMatch(/^Error: should be caught/);
+    }
+  );
+
+
+  it('should not propagate exceptions in success callback to the returned promise', function() {
+    var successCallbackSpy = jasmine.createSpy('successCallback').and.throwError('error');
+    var promiseResolveSpy = jasmine.createSpy('promiseResolve');
+    var promiseRejectSpy = jasmine.createSpy('promiseReject');
+
+    $httpBackend.expectGET('/CreditCard/123').respond(null);
+    var CreditCard = $resource('/CreditCard/:id');
+    CreditCard.get({id: 123}, successCallbackSpy).
+      $promise.then(promiseResolveSpy, promiseRejectSpy);
+
+    $httpBackend.flush();
+    expect(successCallbackSpy).toHaveBeenCalled();
+    expect(promiseResolveSpy).toHaveBeenCalledWith(jasmine.any(CreditCard));
+    expect(promiseRejectSpy).not.toHaveBeenCalled();
+  });
+
+
+  it('should not be able to recover from inside the error callback', function() {
+    var errorCallbackSpy = jasmine.createSpy('errorCallback').and.returnValue({id: 123});
+    var promiseResolveSpy = jasmine.createSpy('promiseResolve');
+    var promiseRejectSpy = jasmine.createSpy('promiseReject');
+
+    $httpBackend.expectGET('/CreditCard/123').respond(404);
+    var CreditCard = $resource('/CreditCard/:id');
+    CreditCard.get({id: 123}, noop, errorCallbackSpy).
+      $promise.then(promiseResolveSpy, promiseRejectSpy);
+
+    $httpBackend.flush();
+    expect(errorCallbackSpy).toHaveBeenCalled();
+    expect(promiseResolveSpy).not.toHaveBeenCalled();
+    expect(promiseRejectSpy).toHaveBeenCalledWith(jasmine.objectContaining({status: 404}));
+  });
+
+
+  describe('requestInterceptor', function() {
+    var rejectReason = {'lol':'cat'};
+    var $q, $rootScope;
+    var successSpy, failureSpy, callback;
+
+    beforeEach(inject(function(_$q_, _$rootScope_) {
+      $q = _$q_;
+      $rootScope = _$rootScope_;
+
+      successSpy = jasmine.createSpy('successSpy');
+      failureSpy = jasmine.createSpy('failureSpy');
+      callback = jasmine.createSpy();
+    }));
+
+    it('should call requestErrorInterceptor if requestInterceptor throws an error', function() {
+      var CreditCard = $resource('/CreditCard', {}, {
+        query: {
+          method: 'get',
+          isArray: true,
+          interceptor: {
+            request: function() {
+              throw rejectReason;
+            },
+            requestError: function(rejection) {
+              callback(rejection);
+              return $q.reject(rejection);
+            }
+          }
+        }
+      });
+
+      var resource = CreditCard.query();
+      resource.$promise.then(successSpy, failureSpy);
+      $rootScope.$apply();
+
+      expect(callback).toHaveBeenCalledOnce();
+      expect(callback).toHaveBeenCalledWith(rejectReason);
+      expect(successSpy).not.toHaveBeenCalled();
+      expect(failureSpy).toHaveBeenCalledOnce();
+      expect(failureSpy).toHaveBeenCalledWith(rejectReason);
+
+      // Ensure that no requests were made.
+      $httpBackend.verifyNoOutstandingRequest();
+    });
+
+    it('should abort the operation if a requestErrorInterceptor throws an exception', function() {
+      var CreditCard = $resource('/CreditCard', {}, {
+        query: {
+          method: 'get',
+          isArray: true,
+          interceptor: {
+            request: function() {
+              return $q.reject();
+            },
+            requestError: function() {
+              throw rejectReason;
+            }
+          }
+        }
+      });
+
+      var resource = CreditCard.query();
+      resource.$promise.then(successSpy, failureSpy);
+      $rootScope.$apply();
+
+      expect(resource.$resolved).toBeTruthy();
+      expect(successSpy).not.toHaveBeenCalled();
+      expect(failureSpy).toHaveBeenCalledOnce();
+      expect(failureSpy).toHaveBeenCalledWith(rejectReason);
+
+      // Ensure that no requests were made.
+      $httpBackend.verifyNoOutstandingRequest();
+    });
+  });
 });
 
 describe('cancelling requests', function() {
@@ -1598,7 +2322,7 @@ describe('cancelling requests', function() {
   );
 
   it('should use `cancellable` value if passed a non-numeric `timeout` in an action',
-    inject(function($log, $q) {
+    inject(function($log, $q, $rootScope) {
       spyOn($log, 'debug');
       $httpBackend.whenGET('/CreditCard').respond({});
 
@@ -1611,6 +2335,7 @@ describe('cancelling requests', function() {
       });
 
       var creditCard = CreditCard.get();
+      $rootScope.$digest();
       expect(creditCard.$cancelRequest).toBeDefined();
       expect(httpSpy.calls.argsFor(0)[0].timeout).toEqual(jasmine.any($q));
       expect(httpSpy.calls.argsFor(0)[0].timeout.then).toBeDefined();
@@ -1709,7 +2434,8 @@ describe('cancelling requests', function() {
       }
     });
 
-    CreditCard.get();
+    var ccs = CreditCard.get();
+    ccs.$promise.catch(noop);
     $timeout.flush();
     expect($httpBackend.flush).toThrow(new Error('No pending request to flush !'));
 
@@ -1728,7 +2454,8 @@ describe('cancelling requests', function() {
       }
     });
 
-    CreditCard.get().$cancelRequest();
+    var ccs = CreditCard.get();
+    ccs.$cancelRequest();
     expect($httpBackend.flush).toThrow(new Error('No pending request to flush !'));
 
     CreditCard.get();
@@ -1746,7 +2473,8 @@ describe('cancelling requests', function() {
       }
     });
 
-    CreditCard.get().$cancelRequest();
+    var ccs = CreditCard.get();
+    ccs.$cancelRequest();
     expect($httpBackend.flush).toThrow(new Error('No pending request to flush !'));
 
     CreditCard.get();
@@ -1771,6 +2499,25 @@ describe('cancelling requests', function() {
 
     expect(creditCard.$cancelRequest).toBe(noop);
   });
+
+  it('should not break when calling old `$cancelRequest` after the response arrives', function() {
+    $httpBackend.whenGET('/CreditCard').respond({});
+
+    var CreditCard = $resource('/CreditCard', {}, {
+      get: {
+        method: 'GET',
+        cancellable: true
+      }
+    });
+
+    var creditCard = CreditCard.get();
+    var cancelRequest = creditCard.$cancelRequest;
+
+    $httpBackend.flush();
+
+    expect(cancelRequest).not.toBe(noop);
+    expect(cancelRequest).not.toThrow();
+  });
 });
 
 describe('configuring `cancellable` on the provider', function() {
